@@ -1,7 +1,7 @@
 
 """each attribute indicates a supported module or feature."""
     
-def supported(mod):
+def module_exists(mod):
     try:
         __import__(mod)
     except ImportError:
@@ -9,5 +9,6 @@ def supported(mod):
     else:
         return True
 
-sqlobject = supported('sqlobject')
-sqlalchemy = supported('sqlalchemy')
\ No newline at end of file
+sqlobject = module_exists('sqlobject')
+sqlalchemy = module_exists('sqlalchemy')
+elixir = module_exists('elixir')
\ No newline at end of file
